require 'uri'

module Artifact

  module Helpers

    #########################################
    # main
    #########################################

    def app_name
      'Artifact'
    end

    def action
      request.path_info.gsub('/','').empty? ? 'home' : request.path_info.gsub('/',' ')
    end

    def requires!(required, hash = params)
      if required.is_a?(Hash)
        required.each { |k, vals| requires!(vals, hash[k]) }
      elsif hash.nil? or hash.is_a?(String) or required.any?{ |p| hash[p].nil? }
        halt(400, 'Missing parameters.')
      end
    end

    #########################################
    # rendering
    #########################################

    def partial(name, *args)
      partial_name = name.to_s["/"] ? name.to_s.reverse.sub("/", "_/").reverse : "_#{name}"
      erb(partial_name.to_sym, {:layout => false}, *args)
    end

    def view(view_name, *args)
      layout = request.xhr? ? false : @mobile_host ? :'layout.mobile' : true
      erb(view_name.to_sym, {:layout => layout}, *args)
    end

    #########################################
    # paths, links
    #########################################

    def get_class_name(klass)
      klass.name.to_s.split("::").last
    end

    def guess_model_path(m)
      '/' + get_class_name(m).pluralize.downcase
    end

    def url_for(object)
      object.nil? ? "#" : object.is_a?(String) ? object \
      : object.respond_to?(:path) ? url(object.path) : url("#{guess_model_path(object.class)}/#{object.to_param}")
    end

    def link_to(string, resource)
      "<a href='#{url_for(resource)}'>#{string}</a>"
    end

    def active?(link)
      request.path[link] ? 'active' : ''
    end

    def upload_relative_path(file)
      # we need to start with a /, otherwise middleman will insert his image_path as prefix
      '/' + Artifact.config.uploads_path + file.path.sub(Artifact.uploads.path, '')
    end

    def is_image?(filename)
      filename.match(/\.(png|jpg|jpeg|gif|bmp)$/i)
    end

    def post_preview_path(post)
      str = post.path.sub(Artifact.config.source_root, '')
      if Artifact.config.directory_indexes
        str.sub(post.filename, post.filename.split('.').first)
      end
    end

    def delete_link(resource, options = {})
      css_class = options[:class] || ''
      text = options[:text] || 'Delete'
      path = url_for(resource)
      '<form class="destroy" style="display:inline" method="post" action="'+ path +'" onsubmit="return confirm(\'Are you sure?\');">
        <input type="hidden" name="_method" value="delete" />
        <button type="submit" class="' + css_class +' btn btn-danger">' + text + '</button>
      </form>'
    end

    def in_words(time)
      minutes = (((Time.now - time).abs)/60).round
      return nil if minutes < 0

      case minutes
        when 0..1            then 'less than a minute'
        when 2..4            then 'less than 5 minutes'
        when 5..14           then 'less than 15 minutes'
        when 15..29          then "half an hour"
        when 30..59          then "#{minutes} minutes"
        when 60..119         then '1 hour'
        when 120..239        then '2 hours'
        when 240..479        then '4 hours'
        when 480..719        then '8 hours'
        when 720..1439       then '12 hours'
        when 1440..11519     then "#{(minutes/1440).floor} days"
        when 11520..43199    then "#{(minutes/11520).floor} weeks"
        when 43200..525599   then "#{(minutes/43200).floor} months"
        else                      "#{(minutes/525600).floor} years"
      end
    end

    def time_ago_in_words(time)
      if str = in_words(time)
        "#{str} ago"
      end
    end

  end

end
