require 'rack/static'

module Artifact

  class Static

    SUFFIXES  = ['', '.html', '/index.html']
    NOT_FOUND = 'Not found mister.'

    DEFAULTS = { # for static
      :root      => 'build',
      :urls      => ['/'],
    }

    def initialize(options = {})
      not_found  = options.delete(:not_found) || NOT_FOUND
      text       = IO.read(not_found) if File.exist?(not_found)

      @suffixes  = options.delete(:suffixes)  || SUFFIXES

      @not_found = [404, {"Content-Type" => 'text/html'}, [text]]
      @static    = ::Rack::Static.new( lambda { [404, {}, []] }, DEFAULTS.merge(options))
    end

    def call(env)
      # resp = Rack::Head.call(env)
      # return resp if resp[0] != 404

      found = nil
      @suffixes.each do |suffix|
       url = env['PATH_INFO'] + suffix
       resp = @static.call(env.merge({'PATH_INFO' => url}))
       break if resp[0] != 404 && found = resp
      end

      found or @not_found
    end

  end

end
